--[[
local profiler = Profiler.new()
function onMouseDown()
	if profiler.displayed then
		profiler:hide()
	else
		profiler:show()
	end
end
stage:addEventListener(Event.MOUSE_DOWN, onMouseDown)
]]

Profiler = Core.class(Sprite)

function Profiler:init()
	self.history = 60
	self.fpsList = {}
	self.memList = {}
	self.started = false
	self.displayed = false
	self.fpsChart = Shape.new()
	self.fpsChart:setX(65)
	self.fpsChart:setY(45)
	self.memChart = Shape.new()
	self.memChart:setX(65)
	self.memChart:setY(90)
	self.bgBox = Shape.new()
	self.bgBox:setFillStyle(Shape.SOLID, 0x000000, 0.5)
	self.bgBox:beginPath()
	self.bgBox:moveTo(0, 0)
	self.bgBox:lineTo(application:getContentWidth(), 0)
	self.bgBox:lineTo(application:getContentWidth(), 120)
	self.bgBox:lineTo(0, 120)
	self.bgBox:closePath()
	self.bgBox:endPath()
	self.bgBox:setFillStyle(Shape.NONE)
	self.bgBox:setLineStyle(1, 0xffffff, 0.2)
	self.bgBox:beginPath()
	self.bgBox:moveTo(65, 45)
	self.bgBox:lineTo(65, 10)
	self.bgBox:moveTo(65, 45)
	self.bgBox:lineTo(application:getContentWidth() - 15, 45)
	self.bgBox:endPath()
	self.bgBox:beginPath()
	self.bgBox:moveTo(65, 90)
	self.bgBox:lineTo(65, 55)
	self.bgBox:moveTo(65, 90)
	self.bgBox:lineTo(application:getContentWidth() - 15, 90)
	self.bgBox:endPath()
	self.infoTxtBx = TextField.new(TTFont.new("arial.ttf", 9))
	self.infoTxtBx:setTextColor(0xcccccc)
	self.infoTxtBx:setY(108)
	local tf = TTFont.new("arial.ttf", 9)
	self.minFpsTxtBx = TextField.new(tf)
	self.minFpsTxtBx:setTextColor(0xaaaaaa)
	self.minFpsTxtBx:setY(47)
	self.maxFpsTxtBx = TextField.new(tf)
	self.maxFpsTxtBx:setTextColor(0xaaaaaa)
	self.maxFpsTxtBx:setY(15)
	self.minMemTxtBx = TextField.new(tf)
	self.minMemTxtBx:setTextColor(0xaaaaaa)
	self.minMemTxtBx:setY(93)
	self.maxMemTxtBx = TextField.new(tf)
	self.maxMemTxtBx:setTextColor(0xaaaaaa)
	self.maxMemTxtBx:setY(60)
	self:addChild(self.bgBox)
	self:addChild(self.infoTxtBx)
	self:addChild(self.minFpsTxtBx)
	self:addChild(self.maxFpsTxtBx)
	self:addChild(self.minMemTxtBx)
	self:addChild(self.maxMemTxtBx)
	self:addChild(self.fpsChart)
	self:addChild(self.memChart)
	self:start()
end

function Profiler:start()
	if self.started then return end
	self.started = true
	self.minFps = 2^24
	self.maxFps = -2^24
	self.minMem = 2^24
	self.maxMem = -2^24
	self.itvTime = os.timer()
	self.initTime = os.timer()
	self.frameCount = 0
	self.totalCount = 0
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
end

function Profiler:stop()
	if not self.started then return end
	self.started = false
	self:removeEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
end

function Profiler:show()
	if self.displayed then return end
	self.displayed = true;
	stage:addChild(self)
end

function Profiler:hide()
	if not self.displayed then return end
	self.displayed = false;
	stage:removeChild(self)
end

function Profiler:onEnterFrame()
	local currentTime = os.timer()
	self.frameCount = self.frameCount + 1
	self.totalCount = self.totalCount + 1
	if currentTime - self.itvTime > 1 then
		local currentFps = self.frameCount / (currentTime - self.itvTime)
		local averageFps = self.totalCount / (currentTime - self.initTime)
		local currentMem = (collectgarbage("count") + application:getTextureMemoryUsage()) * 0.001
		self.minFps = math.min(currentFps, self.minFps)
		self.maxFps = math.max(currentFps, self.maxFps)
		self.minMem = math.min(currentMem, self.minMem)
		self.maxMem = math.max(currentMem, self.maxMem)
		self.infoTxtBx:setText(string.format("Current Fps %.3f | Average Fps %.3f | Memory Used %.3f Mb", currentFps, averageFps, currentMem))
		self.infoTxtBx:setX(application:getContentWidth() - self.infoTxtBx:getWidth() - 20)
		self.minFpsTxtBx:setText(string.format("%.3f Fps", self.minFps))
		self.minFpsTxtBx:setX(60 - self.minFpsTxtBx:getWidth())
		self.maxFpsTxtBx:setText(string.format("%.3f Fps", self.maxFps))
		self.maxFpsTxtBx:setX(60 - self.maxFpsTxtBx:getWidth())
		self.minMemTxtBx:setText(string.format("%.3f Mb", self.minMem))
		self.minMemTxtBx:setX(60 - self.minMemTxtBx:getWidth())
		self.maxMemTxtBx:setText(string.format("%.3f Mb", self.maxMem))
		self.maxMemTxtBx:setX(60 - self.maxMemTxtBx:getWidth())
		local height = 35
		local width = application:getContentWidth() - 80
		local inc = width / (self.history - 1)
		local rateRange = self.maxFps - self.minFps
		self.fpsChart:clear()
		self.fpsChart:setLineStyle(1, 0x33ff00, 0.7)
		self.fpsChart:beginPath()
		for i = 1, #self.fpsList do
			local value = (self.fpsList[i] - self.minFps) / rateRange
			if i == 1 then
				self.fpsChart:moveTo(0, -value * height)
			else
				self.fpsChart:lineTo(i * inc, -value * height)
			end
		end
		self.fpsChart:endPath()
		rateRange = self.maxMem - self.minMem
		self.memChart:clear()
		self.memChart:setLineStyle(1, 0x00aaff, 0.7)
		self.memChart:beginPath()
		for i = 1, #self.memList do
			local value = (self.memList[i] - self.minMem) / rateRange
			if i == 1 then
				self.memChart:moveTo(0, -value * height)
			else
				self.memChart:lineTo(i * inc, -value * height)
			end
		end
		self.memChart:endPath()
		table.insert(self.fpsList, 1, currentFps)
		table.insert(self.memList, 1, currentMem)
		if #self.fpsList > self.history then table.remove(self.fpsList, #self.fpsList) end
		if #self.memList > self.history then table.remove(self.memList, #self.memList) end
		self.itvTime = currentTime
		self.frameCount = 0
	end
end
